function [z, varargout] = rkklein(x, y, r0, a, r1, r2, str)

% RKKLEIN rkklein (see KERATOTOMY)
% 
%   z = rkklein(x, y, r0, a, r1, r2)
% 
% where 
%
%       (x,y)     = position in aperture 
%       r0        = base parabola/sphere 
%       a         = undulation amplitude
%       r1        = inner undulation radius
%       r2        = outer undulation radius 
%
% if r0 < 0 then the results will correspond to :
%   -rkklein(x, y, abs(r0), a, r1, r2)
%
% implements the "klein cornea" equation, 
% 
%  function output=normals(params, a),
%  r=params(1);phi=params(2);                          % location of the corneal point 
%  r0=7.5;r1=1.5;r2=3;del=(r2-r1)/pi;                  % parameters specifying corneal shape 
%  radial=(r>r1)*(r<r2)*(1-cos((r-r1)/del))/2+(r>=r2); % radial function R(r)
%  output(4)=r0-sqrt(r0^2-r^2)+a*cos(8*phi)*radial;    % corneal height 
%  dradial_dr=(r>r1)*(r<r2)*sin((r-r1)/del)/2/del;     % derivative of R(r)
%  output(1)=r/sqrt(r0^2-r^2)+a*cos(8*phi)*dradial_dr; % slope in radial direction
%  output(2)=a*8*sin(8*phi)*radial/r;                  % slope in tangential direction
%  output(3)=sqrt(output(1)^2+output(2)^2);            % total slope (the corneal slant)
%  return 
%


if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
            z = { 7.5, 0.0125, 1.5, 3 };
            varargout = { { 'r0','a','r1','r2' } };
        case { 'inline' },
            z = 'r0-sqrt(r0^2-rx^2)+a*cos(8*phi)*radial';    % corneal height 
            z = strrep(z, 'radial', '((rx>r1)*(rx<r2)*(1-cos((rx-r1)/del))/2+(rx>=r2))');
            z = strrep(z, 'del', '((r2-r1)/pi)');
            z = strrep(z, 'rx', 'sqrt(x^2+y^2)');
            z = strrep(z, 'phi', '(atan2(y,x))');
            z = inline(z,'x','y','r0','a','r1','r2');
        otherwise,
            error('(RKKLEIN) Unknown invocation parameter.');
    end
    return
end


[phi, r]  = cart2pol(x,y); del=(r2-r1)/pi;

if (nargin == 6),
    % default return is the height
    f1 = sign(r0); r0 = abs(r0); % handle r0<0
    radial=(r>r1).*(r<r2).*(1-cos((r-r1)/del))/2+(r>=r2); % radial function R(r)
    z = r0-sqrt(r0^2-r.^2)+a*cos(8*phi).*radial;          % corneal height 
    z = f1*z;  % handle r0<0 
    return
end

switch(str),

    case { 'z' } 
        f1 = sign(r0); r0 = abs(r0); % handle r0<0
        radial=(r>r1).*(r<r2).*(1-cos((r-r1)/del))/2+(r>=r2); % radial function R(r)
        z = r0-sqrt(r0^2-r.^2)+a*cos(8*phi).*radial;          % corneal height 
        z = f1*z;  % handle r0<0
        return
    
    case { 'dzdr' }
        f1 = sign(r0); r0 = abs(r0); % handle r0<0
        dradial_dr=(r>r1).*(r<r2).*sin((r-r1)/del)/2/del;     % derivative of R(r)
        z = r./sqrt(r0^2-r.^2)+a*cos(8*phi).*dradial_dr;         % slope in radial direction
        z = f1*z;  % handle r0<0
        return
    
    case { 'dzdt' }
        f1 = sign(r0); r0 = abs(r0); % handle r0<0
        radial=(r>r1).*(r<r2).*(1-cos((r-r1)/del))/2+(r>=r2); % radial function R(r)
        z = -a*8*sin(8*phi).*radial;                          % bugfix 26/1/07 slope in tangential
        z = f1*z;  % handle r0<0
        return 
        
    case { 'dzdx' }
        gr      = rkklein(x, y, r0, a, r1, r2, 'dzdr');
        gt      = rkklein(x, y, r0, a, r1, r2, 'dzdt');  
        z       = dpol2dcar(phi, r, gt, gr, 'dzdx');        
        return
        
    case { 'dzdy' }
        gr      = rkklein(x, y, r0, a, r1, r2, 'dzdr');
        gt      = rkklein(x, y, r0, a, r1, r2, 'dzdt');  
        z       = dpol2dcar(phi, r, gt, gr, 'dzdy');        
        return
        
    case { 'normal' }
        gr      = rkklein(x, y, r0, a, r1, r2, 'dzdr');
        gt      = rkklein(x, y, r0, a, r1, r2, 'dzdt');    
        z(:,1)  = dpol2dcar(phi, r, gt, gr, 'dzdx');        
        z(:,2)  = dpol2dcar(phi, r, gt, gr, 'dzdy');        
        z(:,3)  = -1;
        return
        
    otherwise 
        error('(rkklein) unknown option requested.');
        
end    

return

