function [z, varargout] = rksicam(x, y, r0, a, r1, r2, str)

% RKSICAM radial keratotomy model 
% used by Sicam et al. adapted from Rand et al. 
% 
%   z = rksicam(x, y, r0, a, r1, r2, str)
% 
% where 
%       (x,y)     = position in aperture 
%       r0        = base parabola/sphere 
%       a         = undulation amplitude
%       r1        = inner undulation radius
%       r2        = outer undulation radius 
%
% see Corneal surface reconstruction algorithm that uses 
% Zernike polynomial representation. 
% Sicam, V.A.; Coppens, J.; van den Berg, Tom J. T. P. and 
% van der Heijde, Rob G. L.


if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
            z = { 7, 0.01, 1.5, 2.0 };
            varargout = { { 'r0','a','r1','r2' } };
        case { 'inline' },
            z   = '(rx^2/(2*r0))  + (a*sin(8*phi)*(rx>=r1)) + (2*a*(rx-r1)*sin(8*phi)*((rx<r2)&(rx>r1)))';
            z = strrep(z, 'radial', '((rx>r1)*(rx<r2)*(1-cos((rx-r1)/del))/2+(rx>=r2))');
            z = strrep(z, 'del', '((r2-r1)/pi)');
            z = strrep(z, 'rx', 'sqrt(x^2+y^2)');
            z = strrep(z, 'phi', '(atan2(y,x))');
            z = inline(z,'x','y','r0','a','r1','r2');
        otherwise,
            error('(RKSICAM) Unknown invocation parameter.');
    end
    return
end

[t, r]  = cart2pol(x,y);

if  (nargin == 6),
    z   = a*sin(8*t).*(r>=r2) + ((r-r1)/(r2-r1)).*(a*sin(8*t)).*((r<r2)&(r>r1));  
    z   = r.^2/(2*r0) + z;
    return
end 

switch(str)
    case { 'z' }, 
        z   = a*sin(8*t).*(r>=r2) + ((r-r1)/(r2-r1)).*(a*sin(8*t)).*((r<r2)&(r>r1));  
        z   = r.^2/(2*r0) + z;
    case { 'dzdr' },
        z   = (1/(r2-r1))*(a*sin(8*t)).*((r<r2)&(r>r1));  
        z   = z  + r/r0;
    case { 'dzdt' }
        z   = (a*8*cos(8*t)).*(r>=r2) + ((r-r1)/(r2-r1)).*(a*8*cos(8*t)).*((r<r2)&(r>r1)); 
    case { 'dzdx' }
        gr   = rksicam(x, y, r0, a, r1, r2, 'dzdr');
        gt   = rksicam(x, y, r0, a, r1, r2, 'dzdt');  
        z    = dpol2dcar(t, r, gt, gr, 'dzdx');        
    case { 'dzdy' }
        gr   = rksicam(x, y, r0, a, r1, r2, 'dzdr');
        gt   = rksicam(x, y, r0, a, r1, r2, 'dzdt');  
        z    = dpol2dcar(t, r, gt, gr,'dzdy');        
    case { 'normal' }
        gr   = rksicam(x, y, r0, a, r1, r2, 'dzdr');
        gt   = rksicam(x, y, r0, a, r1, r2, 'dzdt');  
        z(:,1)  = dpol2dcar(t, r, gt, gr,'dzdx');        
        z(:,2)  = dpol2dcar(t, r, gt, gr,'dzdy');        
        z(:,3)  = -1;
    otherwise 
        error('unknown option requested.');
end    
return

